<?php

class MachII_framework_Config
{

    var $xmlFile;
    
    var $parser;

    var $depth = 0;
    
    var $elements = array();
    
    var $element;
    
    var $listener;
    
    var $eventHandler;
    
    var $plugin;
    
    var $config = array(
        'properties'     => array(),
        'listeners'      => array(),
        'event-filters'  => array(),
        'event-handlers' => array(),
        'page-views'     => array(),
        'plugins'        => array()
    );
    
    
    
    function MachII_framework_Config()
    {
        $parser =& xml_parser_create();
        $this->parser =& $parser;
        xml_set_object($this->parser, $this);
        xml_parser_set_option($this->parser, XML_OPTION_CASE_FOLDING, false);
        xml_set_element_handler($this->parser, '_openElementHandler', '_closeElementHandler');
    }
    
    function &loadConfig($file)
    {
        $xmlFile = fopen($file, 'rb');
        if ($xmlFile === false) {
            $error =& MachII::throwError(
                array('message' => $file),
                MACHII_ERROR_CONFIG_LOAD
            );
            return $error;
        }
        $this->xmlFile =& $xmlFile;
        return MACHII_OK;
    }
    
    function &parse($file)
    {
        $error =& $this->loadConfig($file);
        if (MachII::isError($error)) {
            return $error;
        }
        while ($xml = fread($this->xmlFile, 2048)) {
            if (!$this->parseString($xml, feof($this->xmlFile))) {
                $error =& MachII::throwError(
                    array(
                        'lineNo'   => xml_get_current_line_number($this->parser),
                        'columnNo' => xml_get_current_column_number($this->parser),
                        'message'  => xml_error_string(xml_get_error_code($this->parser))
                    ),
                    MACHII_ERROR_CONFIG_PARSE
                );
                return $error;
            }
        }
        
        fclose($this->xmlFile);
        xml_parser_free($this->parser);
        
        return MACHII_OK;
    }
    
    
    function parseString($xml, $eof = false)
    {
        return xml_parse($this->parser, $xml, $eof);
    }
    
    
    function _openElementHandler($parser, $element, $attribs)
    {
        //event-handler count
        static $ehCnt;
        
        $this->elements[$this->depth++] = strtolower($element);
        $this->element = implode('/', $this->elements);
        
        switch ($this->element) {
            case 'mach-ii/properties/property':
               $this->config['properties'][$attribs['name']] = $attribs['value'];
                break;
            
            case 'mach-ii/listeners/listener':
                $this->listener = $attribs['name'];
                $this->config['listeners'][$this->listener] =  $attribs;
                $this->config['listeners'][$this->listener]['parameters'] = array();
                break;
                
            case 'mach-ii/listeners/listener/invoker':
                $this->config['listeners'][$this->listener]['invoker'] = $attribs;
                break;
                
            case 'mach-ii/listeners/listener/parameters/parameter':
                $this->config['listeners'][$this->listener]['parameters'][$attribs['name']] = $attribs['value'];
                break;
                
            case 'mach-ii/event-filters/event-filter':
                $this->eventFilter = $attribs['name'];
                $this->config['event-filters'][$this->eventFilter] = $attribs;
                break;
                
            case 'mach-ii/event-handlers/event-handler':
                $ehCnt = -1;
                $access = (!empty($attribs['access'])
                    && (strtolower($attribs['access']) == 'private'))?
                        MACHII_EVENT_ACCESS_PRIVATE:
                        MACHII_EVENT_ACCESS_PUBLIC;
                $this->eventHandler = $attribs['event'];
                $this->config['event-handlers'][$attribs['event']]['access'] = $access;
                break;
                
            case 'mach-ii/event-handlers/event-handler/filter':
                $this->config['event-handlers'][$this->eventHandler][++$ehCnt] = array(
                    'filter' => array_merge($attribs, array('parameters' => array())));    
                break;
                
            case 'mach-ii/event-handlers/event-handler/filter/parameter':
                $this->config['event-handlers'][$this->eventHandler][$ehCnt]['filter']['parameters'][$attribs['name']] = $attribs['value'];
                break;
                
            case 'mach-ii/event-handlers/event-handler/notify':
                if (empty($attribs['resultKey'])) {
                    $attribs['resultKey'] = null;
                } 
                $this->config['event-handlers'][$this->eventHandler][++$ehCnt] = array(
                    'notify' => $attribs
                );
                break;
                
            case 'mach-ii/event-handlers/event-handler/event-arg':
                if (empty($attribs['value'])) {
                    $attribs['value'] = null;
                }
                if (empty($attribs['variable'])) {
                    $attribs['variable'] = null;
                }
                $this->config['event-handlers'][$this->eventHandler][++$ehCnt] = array(
                    'event-arg' => $attribs
                );
                break;
                
            case 'mach-ii/event-handlers/event-handler/event-mapping':
                $this->config['event-handlers'][$this->eventHandler][++$ehCnt] = array(
                    'event-mapping' => $attribs
                );
                break;
                
            case 'mach-ii/event-handlers/event-handler/view-page':
                if (empty($attribs['contentKey'])) {
                    $attribs['contentKey'] = null;
                }
                $this->config['event-handlers'][$this->eventHandler][++$ehCnt] = array(
                    'view-page' => $attribs
                );
                break;
                
            case 'mach-ii/event-handlers/event-handler/announce':
                $attribs['copyEventArgs'] = 
                    (isset($attribs['copyeventargs'])
                    && ($attribs['copyeventargs'] == 'false'))?
                        false:true;
                $this->config['event-handlers'][$this->eventHandler][++$ehCnt] = array(
                    'announce' => $attribs
                );
                break;
                
            case 'mach-ii/event-handlers/event-handler/event-bean':
                if (empty($attribs['fields'])) {
                    $attribs['fields'] = null;
                }
                $this->config['event-handlers'][$this->eventHandler][++$ehCnt] = array(
                    'event-bean' => $attribs
                );
                break;
            
            case 'mach-ii/page-views/page-view':
                $this->config['page-views'][$attribs['name']] = $attribs['page'];
                break;
                
            case 'mach-ii/plugins/plugin':
                $this->plugin = $attribs['name'];
                $this->config['plugins'][$this->plugin] = $attribs;
                $this->config['plugins'][$this->plugin]['parameters'] = array();
                break;
                
            case 'mach-ii/plugins/plugin/parameters/parameter':
                $this->config['plugins'][$this->plugin]['parameters'][$attribs['name']] = $attribs['value'];
        }
    }
    
    
    function _closeElementHandler($parser, $element)
    {
        if ($this->element == 'mach-ii') {
            // Done; do some cleanup.
            unset($this->listener);
            unset($this->eventFilter);
            unset($this->plugin);
        }
        
        unset($this->elements[--$this->depth]);
        $this->element = implode('/', $this->elements);
    }
    
    
    function getConfig($element)
    {
        return (isset($this->config[$element]))?$this->config[$element]:false;
    }

}

?>