<?php
/**
 * @package MachII
 */
/**
 * EventFilter class.
 * @package MachII
 */
class MachII_framework_EventFilter
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    var $_appManager;
    /**
     * @var array associative array of filter parameters
     * @access private
     */
    var $_parameters = array();
    
    
    /**
     * 
     */
    function MachII_framework_EventFilter(&$appManager, $parameters)
    {
        $this->_setAppManager($appManager);
        $this->setParameters($parameters);
    }
    
    /**
     * @abstract
     */
    function configure() {}
    
    /**
     * @param MachII_framework_Event or descendent
     * @param MachII_framework_EventContext
     * @param array associative array of filter parameters
     * @abstract
     */
    function filterEvent(&$event, &$eventContext, $paramArgs = array())
    {
        return true;
    }
    
    /**
     * @access private
     * @param MachII_framework_AppManager
     */
    function _setAppManager(&$appManager)
    {
        $this->_appManager =& $appManager;
    }
    
    /**
     * @access protected
     * @return MachII_framework_AppManager
     */
    function &_getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param string
     * @param mixed
     */
    function setParameter($name, $value)
    {
        $this->_parameters[$name] = $value;
    }
    
    /**
     * @param string
     * @param mixed
     * @return mixed
     */
    function getParameter($name, $defaultValue = null)
    {
        return (isset($this->_parameters[$name]))?$this->_parameters[$name]:$defaultValue;
    }
    
    /**
     * @param string
     * @return boolean
     */
    function isParameterDefined($name)
    {
        return isset($this->_parameters[$name]);
    }
    
    /**
     * @param array associative array will be merged with any existing parameters.
     */
    function setParameters($parameters)
    {
        $this->_parameters = array_merge($this->_parameters, $parameters);
    }
    
    /**
     * @return array associative array
     */
    function getParameters()
    {
        return $this->_parameters;
    }

}
?>