<?php
/**
 * @package MachII
 */
/**
 * Listener.
 * @package MachII
 */
class MachII_framework_Listener
{
    /**
     * Dispatch map.
     * 
     * Provides a map from named arguments to listener method arguments.
     * Named as it is to be compatible/consistent with PEAR {@link http://pear.php.net/package/SOAP SOAP}.
     * 
     * Example:
     * <code>
     * 
     * </code>
     * @var array
     * @see $__typedef
     * @access protected
     */
    var $__dispatch_map = array();
    
    /**
     * Variable type definitions.
     * 
     * Provides a map of method variable types. 
     * Named as it is to be compatible/consistent with PEAR {@link http://pear.php.net/package/SOAP SOAP}. 
     * @var array
     * @see $__dispatch_map
     * @access protected
     */
    var $__typedef = array();
    
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    var $_appManager;
    
    /**
     * @var MachII_framework_Invoker descendent
     * @access private
     */
    var $_invoker;
    
    /**
     * @var array associative array of listener parameters
     * @access private
     */
    var $_parameters = array();
    
    
    /**
     * @param MachII_framework_AppManager
     * @param array associative array of listener parameters
     * @param MachII_framework_ListenerInvoker descendent
     */
    function MachII_framework_Listener($appManager, $parameters = array(), $invoker = null)
    {
        $this->setAppManager($appManager);
        $this->setParameters($parameters);
        
        if (isset($invoker)) {
            $this->setInvoker($invoker);
        }
        
    }
    
    /**
     * Configure the listener.
     * @abstract
     */
    function configure() {}
    
    /**
     * @param string
     * @param array associative array of event arguments
     */
    function announceEvent($eventName, $eventArgs = array())
    {//echo 'listener::announceevent<br />';
        if (isset($GLOBALS['eventContext'])) {
            $GLOBALS['eventContext']->announceEvent($eventName, $eventArgs);
        }
    }
    
    /**
     * @param MachII_framework_Invoker descendent
     */
    function setInvoker(&$invoker)
    {
        $this->_invoker =& $invoker;
    }
    
    /**
     * @return MachII_framework_Invoker descendent
     */
    function &getInvoker()
    {
        return $this->_invoker;
    }
    
    /**
     * @param string
     */
    function isParameterDefined($name)
    {
        return isset($this->_parameters[$name]);
    }
    
    /**
     * @param string
     * @param mixed
     */
    function setParameter($name, $value)
    {
        $this->_parameters[$name] = $value;
    }
    
    /**
     * @param string
     * @param mixed
     */
    function getParameter($name, $defaultValue = null)
    {
        return (isset($this->_parameters[$name]))?
            $this->_parameters[$name]:
            $defaultValue;
    }
    
    /**
     * @param array associative array of listener parameters will
     *              be merged with any existing parameters.
     */
    function setParameters($parameters)
    {
        $this->_parameters = array_merge($this->_parameters, $parameters);
    }
    
    /**
     * @return array associative array of listener parameters.
     */
    function getParameters()
    {
        return $this->_parameters;
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    function setAppManager(&$appManager)
    {
        $this->_appManager =& $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    function &getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param string
     * @return array associative array of argument mappings.
     */
    function __dispatch($methodName)
    {
        return (isset($this->__dispatch_map[$methodName]))?
            $this->__dispatch_map[$methodName]:
            null;
    }

}
?>