<?php
/**
 * @package MachII
 */
require_once 'MachII/framework/ListenerInvoker.php';
/**
 * Manages registered listeners.
 * @package MachII
 */
class MachII_framework_ListenerManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    var $_appManager;
    
    /**
     * @var array associative array of MachII_framework_ListenerManager
     *            descendents.
     * @access private
     */
    var $_listeners = array();
    
    /**
     * @param array associative array of raw listener configuration data
     * @param MachII_framework_AppManager
     */
    function init($listeners, &$appManager)
    {
        foreach ($listeners as $listener) {
            $this->setAppManager($appManager);
            
            // Setup the listener's invoker
            $invokerType = MachII_util_ClassLoader::load($listener['invoker']['type']);
            $invoker =& new $invokerType();
            
            // Listener
            $listenerType = MachII_util_ClassLoader::load($listener['type']);
            
            $listenerObj =& new $listenerType($appManager, $listener['parameters']);
            $listenerObj->setInvoker($invoker);
            
            $this->addListener($listener['name'], $listenerObj);
        }
    }
    
    /**
     * Configure all of the listeners.
     * @throws MACHII_ERROR*
     */
    function configure()
    {
        $keys = array_keys($this->_listeners);
        foreach ($keys as $key) {
            $listener =& $this->getListener($key);
            if (MachII::isError($listener)) {
                return $listener;
            }
            
            $error = $listener->configure();
            if (MachII::isError($error)) {
                return $error;
            }
        }
    }
    
    /**
     * @param string
     * @param MachII_framework_Listener descendent
     */
    function addListener($name, &$listener)
    {
        $this->_listeners[$name] =& $listener;
    }
    
    /**
     * @param string
     * @return MachII_framework_Listener|false
     */
    function &getListener($listenerName)
    {
        if ($listener = $this->isListenerDefined($listenerName)) {
            return $this->_listeners[$listenerName];
        }
        
        return $listener;
    }
    
    /**
     * @param string
     * @return boolean
     */
    function isListenerDefined($filterName)
    {
        return isset($this->_listeners[$filterName]);
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    function setAppManager(&$appManager)
    {
        $this->_appManager =& $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    function &getAppManager()
    {
        return $this->_appManager;
    }

}

?>