<?php
/**
 * @package MachII
 */
/**
 * Plugin.
 * @package MachII
 */
class MachII_framework_Plugin
{
    /**
     * @var MachII_framework_AppManager
     * @access protected
     */
    var $_appManager;
    
    /**
     * @var array associative array of plugin configuration parameters.
     * @access protected
     */
    var $_parameters = array();
    
    
    /**
     * @param MachII_framework_AppManager
     * @param array associative array of plugin configuration data.
     */
    function MachII_framework_Plugin(&$appManager, $parameters = array())
    {
        $this->_setAppManager($appManager);
        $this->setParameters($parameters);
    }
    
    /**
     * @abstract
     */
    function configure() {}
    
    /**
     * @param MachII_framework_EventContext
     * @param MachII_util_Exception
     */
    function handleException(&$eventContext, &$exception) {}
    
    /**
     * @param string 
     * @throws MACHII_ERROR_ABORT_EVENT_EXCEPTION
     */
    function abortEvent($message = '')
    {
        $error =& MachII::throwError(
            array('message' => $message),
            MACHII_ERROR_ABORT_EVENT_EXCEPTION
        );
        
        return $error;
    }
    
    /**
     * @param string
     * @param mixed
     */
    function setParameter($name, $value)
    {
        $this->_parameters[$name] = $value;
    }
    
    /**
     * @param string
     * @param mixed
     * @return mixed
     */
    function getParameter($name, $defaultValue = null)
    {
        return (isset($this->_parameters[$name]))?
                $this->_parameters[$name]:
                $defaultValue;
    }
    
    /**
     * @param array associative array of plugin parrameters will
     *              be merged with any existing parameters.
     */
    function setParameters($parameters)
    {
        $this->_parameters = array_merge($this->_parameters, $parameters);
    }
    
    /**
     * @return array associative array of plugin parrameters.
     */
    function getParameters()
    {
        return $this->_parameters;
    }
    
    /**
     * @access private
     * @param MachII_framework_AppManager
     */
    function _setAppManager($appManager)
    {
        $this->_appManager =& $appManager;
    }
    
    /**
     * @access protected
     * @return MachII_framework_AppManager
     */
    function &_getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * Executes once before processing events.
     * @param MachII_framework_EventContext
     */
    function preProcess(&$eventContext) {}
    
    /**
     * Executes before each event.
     * @param MachII_framework_EventContext
     */
    function preEvent(&$eventContext) {}
    
    /**
     * Executes after each event.
     * @param MachII_framework_EventContext
     */
    function postEvent(&$eventContext) {}
    
    /**
     * Executes before each view.
     * @param MachII_framework_EventContext
     */
    function preView(&$eventContext) {}
    
    /**
     * Executes after each view.
     * @param MachII_framework_EventContext
     */
    function postView(&$eventContext) {}
    
    /**
     * Executes once after processing events.
     * @param MachII_framework_EventContext
     */
    function postProcess(&$eventContext) {}
    
}

?>