<?php
/**
 * @package MachII
 * @subpackage Commands
 */
/**
 * An EventCommand for creating and populating a bean in the current event.
 * @package MachII
 * @subpackage Commands
 */
class MachII_framework_commands_EventBeanCommand extends MachII_framework_EventCommand
{
    /**
     * @access private
     * @var string
     */
    var $_beanName;
    /**
     * @access private
     * @var string
     */
    var $_beanType;
    /**
     * @access private
     * @var string
     */
    var $_beanFields;
    /**
     * @access private
     * @var BeanUtil
     */
    var $_beanUtil;
    
    /**
     * @param string
     * @param string
     * @param string a comma delimited list
     */
    function MachII_framework_commands_EventBeanCommand($beanName, $beanType, $beanFields)
    {
        $this->setBeanName($beanName);
        $this->setBeanType($beanType);
        $this->setBeanFields($beanFields);
        
        $className = MachII_util_ClassLoader::load('MachII.util.BeanUtil');
        $this->_setBeanUtil(new $className());
    }
    
    /**
     * @param MachII_framework_Event MachII_framework_Event descendent
     * @param MachII_framework_EventContext
     * @return true
     */
    function execute(&$event, &$eventContext)
    {
        $beanUtil =& $this->_getBeanUtil();
        $beanType = $this->getBeanType();
        
        if ($this->isBeanFieldsDefined()) {
            $bean =& $beanUtil->createBean($beanType);
            $eventArgs =& $event->getArgs();
            $beanUtil->setBeanFields($bean, $this->getBeanFields(), $eventArgs);
        } else {
            $bean =& $beanUtil->createBean($beanType, $event->getArgs());
        }
        
        $event->setArg($this->getBeanName(), $bean, $beanType);
        
        return MACHII_OK;
    }
    
    /**
     * @param string
     */
    function setBeanName($beanName)
    {
        $this->_beanName = $beanName;
    }
    
    /**
     * @return string
     */
    function getBeanName()
    {
        return $this->_beanName;
    }
    
    /**
     * @param string
     */
    function setBeanType($beanType)
    {
        $this->_beanType = $beanType;
    }
    
    /**
     * @return string
     */
    function getBeanType()
    {
        return $this->_beanType;
    }
    
    /**
     * @param string a comma delimited list of field names
     */
    function setBeanFields($beanFields)
    {
        $this->_beanFields = $beanFields;
    }
    
    /**
     * @return string a comma delimited list of field names
     */
    function getBeanFields()
    {
        return $this->_beanFields;
    }
    
    /**
     * @return boolean
     */
    function isBeanFieldsDefined()
    {
        return isset($this->_beanFields);
    }
    
    
    /**
     * @access protected
     * @param BeanUtil
     */
    function _setBeanUtil($beanUtil)
    {
        $this->_beanUtil =& $beanUtil;
    }
    
    
    /**
     * @access protected
     * @return BeanUtil
     */
    function &_getBeanUtil()
    {
        return $this->_beanUtil;
    }

}

?>