<?php
/**
 * @package MachII
 */
/**
 * Plugin.
 * @package MachII
 */
abstract class MachII_framework_Plugin
{
    /**
     * @var MachII_framework_AppManager
     * @access protected
     */
    protected $_appManager;
    
    /**
     * @var array associative array of plugin configuration parameters.
     * @access protected
     */
    protected $_parameters = array();
    
    
    /**
     * @param MachII_framework_AppManager
     * @param array associative array of plugin configuration data.
     */
    public function __construct(MachII_framework_AppManager $appManager, $parameters = array())
    {
        $this->_setAppManager($appManager);
        $this->setParameters($parameters);
    }
    
    /**
     * @internal does nothing at this time
     */
    public function configure() {}
    
    /**
     * @param MachII_framework_EventContext
     * @param MachII_util_Exception
     */
    public function handleException(MachII_framework_EventContext $eventContext, MachII_util_Exception $exception) {}
    
    /**
     * @param string 
     * @throws MACHII_ERROR_ABORT_EVENT_EXCEPTION
     */
    public function abortEvent($message = '')
    {
        $error = MachII::throwError(
            array('message' => $message),
            MACHII_ERROR_ABORT_EVENT_EXCEPTION
        );
        
        return $error;
    }
    
    /**
     * @param string
     * @param mixed
     */
    public function setParameter($name, $value)
    {
        $this->_parameters[$name] = $value;
    }
    
    /**
     * @param string
     * @param mixed
     * @return mixed
     */
    public function getParameter($name, $defaultValue = null)
    {
        return (isset($this->_parameters[$name]))?
                $this->_parameters[$name]:
                $defaultValue;
    }
    
    /**
     * @param array associative array of plugin parrameters will
     *              be merged with any existing parameters.
     */
    public function setParameters($parameters)
    {
        $this->_parameters = array_merge($this->_parameters, $parameters);
    }
    
    /**
     * @return array associative array of plugin parrameters.
     */
    public function getParameters()
    {
        return $this->_parameters;
    }
    
    /**
     * @access private
     * @param MachII_framework_AppManager
     */
    public function _setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @access protected
     * @return MachII_framework_AppManager
     */
    public function _getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * Executes once before processing events.
     * @param MachII_framework_EventContext
     */
    public abstract function preProcess(MachII_framework_EventContext $eventContext);
    
    /**
     * Executes before each event.
     * @param MachII_framework_EventContext
     */
    public abstract function preEvent(MachII_framework_EventContext $eventContext);
    
    /**
     * Executes after each event.
     * @param MachII_framework_EventContext
     */
    public abstract function postEvent(MachII_framework_EventContext $eventContext);
    
    /**
     * Executes before each view.
     * @param MachII_framework_EventContext
     */
    public abstract function preView(MachII_framework_EventContext $eventContext);
    
    /**
     * Executes after each view.
     * @param MachII_framework_EventContext
     */
    public abstract function postView(MachII_framework_EventContext $eventContext);
    
    /**
     * Executes once after processing events.
     * @param MachII_framework_EventContext
     */
    public abstract function postProcess(MachII_framework_EventContext $eventContext);
    
}

?>