<?php
/**
 * @package MachII
 */
/**
 * Manages registered views.
 * @package MachII
 */
class MachII_framework_ViewManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of filesystem paths
     * @access private
     */
    private $_viewPaths = array();
    
    
    /**
     * @param array associative array of filesystem paths
     * @param MachII_framework_AppManager
     */
    public function init($pageViews, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        $this->_viewPaths = $pageViews;
    }
    
    /**
     * @internal not used at this time 
     */
    public function configure() {}
    
    /**
     * @param string
     * @throws MACHII_VIEW_NOT_DEFINED
     * @return string
     */
    public function getViewPath($viewName)
    {
    //$GLOBALS['vard']->display($this->_viewPaths);
        if (!$this->isViewPathDefined($viewName)) {
            $error = MachII::throwError(
                array('viewName' => $viewName),
                MACHII_ERROR_VIEW_NOT_DEFINED
            );
            return $error;
        }
        //echo 'viewManager->getViewPath: ',$this->_viewPaths[$viewName],'<br />';
        return $this->_viewPaths[$viewName];
    }
    
    /**
     * @param string
     * @return boolean
     */
    public function isViewPathDefined($viewName)
    {
        return isset($this->_viewPaths[$viewName]);
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }

}

?>