<?php
/**
 * @package MachII
 * @subpackage Commands
 */
/**
 * An EventCommand for announcing an event.
 * @package MachII
 * @subpackage Commands
 */
class MachII_framework_commands_AnnounceCommand extends MachII_framework_EventCommand
{
    /**
     * @var string
     * @access private
     */
    private $_eventName;
    
    /**
     * @var boolean
     * @access private
     */
    private $_copyEventArgs;
    
    
    /**
     * @param string
     * @param boolean
     */
    public function __construct($eventName, $copyEventArgs = true)
    {
        $this->_setEventName($eventName);
        $this->_setCopyEventArgs($copyEventArgs);
    }
    
    /**
     * @param MachII_framework_Event MachII_framework_Event descendent
     * @param MachII_framework_EventContext
     * @throws MACHII_ERROR*
     * @return MACHII_OK
     */
    public function execute(MachII_framework_Event $event, MachII_framework_EventContext $eventContext)
    {
        $eventArgs = array();
        if ($this->_isCopyEventArgs()) {
            $eventArgs =& $event->getArgs();
        }
        
        $error = $eventContext->announceEvent($this->_getEventName(), $eventArgs);
        if (MachII::isError($error)) {
            return $error;
        }
        
        return MACHII_OK;
    }
    
    /**
     * @access private
     * @param string
     */
    private function _setEventName($eventName)
    {
        $this->_eventName = $eventName;
    }
    
    /**
     * @access private
     * @return string
     */
    private function _getEventName()
    {
        return $this->_eventName;
    }
    
    /**
     * @access private
     * @param boolean
     */
    private function _setCopyEventArgs($copyEventArgs)
    {
        $this->_copyEventArgs = $copyEventArgs;
    }
    
    /**
     * @access private
     * @return boolean
     */
    private function _isCopyEventArgs()
    {
        return $this->_copyEventArgs;
    }

}

?>