<?php
/**
 * @package MachII
 * @subpackage Commands
 */
/**
 * An EventCommand for creating and populating a bean in the current event.
 * @package MachII
 * @subpackage Commands
 */
class MachII_framework_commands_EventBeanCommand extends MachII_framework_EventCommand
{
    /**
     * @access private
     * @var string
     */
    private $_beanName;
    /**
     * @access private
     * @var string
     */
    private $_beanType;
    /**
     * @access private
     * @var string
     */
    private $_beanFields;
    /**
     * @access private
     * @var BeanUtil
     */
    private $_beanUtil;
    
    /**
     * @param string
     * @param string
     * @param string a comma delimited list
     */
    public function __construct($beanName, $beanType, $beanFields)
    {
        $this->setBeanName($beanName);
        $this->setBeanType($beanType);
        $this->setBeanFields($beanFields);
        
        $className = MachII_util_ClassLoader::load('MachII.util.BeanUtil');
        $this->_setBeanUtil(new $className());
    }
    
    /**
     * @param MachII_framework_Event MachII_framework_Event descendent
     * @param MachII_framework_EventContext
     * @return true
     */
    public function execute(MachII_framework_Event $event, MachII_framework_EventContext $eventContext)
    {
        $beanType = $this->getBeanType();
        
        if ($this->isBeanFieldsDefined()) {
            $bean = $this->_getBeanUtil()->createBean($beanType);
            $this->_getBeanUtil()->setBeanFields($bean, $this->getBeanFields(), $event->getArgs());
        } else {
            $bean = $beanUtil->createBean($beanType, $event->getArgs());
        }
        
        $event->setArg($this->getBeanName(), $bean, $beanType);
        
        return MACHII_OK;
    }
    
    /**
     * @param string
     */
    public function setBeanName($beanName)
    {
        $this->_beanName = $beanName;
    }
    
    /**
     * @return string
     */
    public function getBeanName()
    {
        return $this->_beanName;
    }
    
    /**
     * @param string
     */
    public function setBeanType($beanType)
    {
        $this->_beanType = $beanType;
    }
    
    /**
     * @return string
     */
    public function getBeanType()
    {
        return $this->_beanType;
    }
    
    /**
     * @param string a comma delimited list of field names
     */
    public function setBeanFields($beanFields)
    {
        $this->_beanFields = $beanFields;
    }
    
    /**
     * @return string a comma delimited list of field names
     */
    public function getBeanFields()
    {
        return $this->_beanFields;
    }
    
    /**
     * @return boolean
     */
    public function isBeanFieldsDefined()
    {
        return isset($this->_beanFields);
    }
    
    
    /**
     * @access private
     * @param BeanUtil
     */
    private function _setBeanUtil($beanUtil)
    {
        $this->_beanUtil = $beanUtil;
    }
    
    
    /**
     * @access private
     * @return BeanUtil
     */
    private function &_getBeanUtil()
    {
        return $this->_beanUtil;
    }

}

?>