<?php
/**
 * @package MachII
 * @subpackage Invokers
 */
/**
 * A MachII_framework_Invoker for calling a MachII_framework_Listener.
 * 
 * Event arguments will be mapped to MachII_framework_Listener method
 * arguments via a manually created argument map.
 * @see MachII_framework_Listener::__dispatch_map
 * @package MachII
 * @subpackage Invokers
 */
class MachII_framework_invokers_ObjectEventArgs extends MachII_framework_ListenerInvoker
{
    /**
     * {@inheritdoc}
     * @uses MachII_framework_Listener::__dispatch
     */
    public function invokeListener(MachII_framework_Event $event, MachII_framework_Listener $target, $method, $resultKey = null)
    {
        //$resultKeyRef = (isset($resultKey))?$resultKey:'resultKeyRef';
        
        // Get the map of inputs for the method being called
        $dispatchMap = $target->__dispatch($method);
        if (!$dispatchMap) {
            $error = MachII::throwError(
                array(
                    'className'  => get_class($target),
                    'methodName' => $method
                ),
                MACHII_ERROR_LISTENER_DISPATCH_MAP_NOT_DEFINED
            );
            return $error;
        } // TODO: maybe use a tokenizer to build map at parse time?
        
        // Build the arg list
        $args = array();
        $eventArgs = $event->getArgs();
        
        $dispatchMapArgs = array_keys($dispatchMap['in']);
        
        foreach ($dispatchMapArgs as $key=>$arg) {
            if (isset($eventArgs[$arg])) {
                $args[] = '$eventArgs[$dispatchMapArgs['.$key.']]';
            }
        }
        if (count($dispatchMap['in']) != count($args)) {
            $error = MachII::throwError(
                array(
                    'className'       => get_class($target),
                    'methodName'      => $method,
                    'argsFoundNum'    => count($args),
                    'argsExpectedNum' => count($dispatchMap['in'])
                ),
                MACHII_ERROR_LISTENER_ARG_COUNT_MISMATCH
            );
            return $error;
        }
        
        //$argStr = implode(', ', $args);
        
        // This would be way better with call_user_func_array() but it can't return a reference.
        // TODO: Might be fixed in PHP5.
        //echo("return \$$resultKeyRef =& \$target->\$method($argStr);<br />");
        //eval("return \$$resultKeyRef = \$target->\$method($argStr);");
        $resultKeyRef = call_user_func_array(array($target, $method), $args);
        
        if ((isset($resultKey))
        && (!strstr($resultKey, '['))) {
            $event->setArg($resultKey, $$resultKeyRef);
        }
        
        if (MachII::isError($resultKeyRef)) {
            return $resultKeyRef;
        }
    }

}

?>