<?php

/*
Author: Ben Edwards (ben@ben-edwards.com)

RequiredFieldsFilter
    This event-filter tests an event for required fields specified.
    If the required fields are not present (or are blank) then event 
    processing is aborted and a specified event is announced.
    
    If the required fields aren't defined then 'message' and 'missingFields' 
    are set in the event.
    
Configuration Parameters:
    None.
    
Event-Handler Parameters:
    "requiredFields" - a comma delimited list of fields required
    "invalidEvent" - the event to announce if all required fields are not in the event
*/

class MachII_filters_RequiredFieldsFilter extends MachII_framework_EventFilter
{

    private $REQUIRED_FIELDS_PARAM = 'requiredFields';
    
    private $INVALID_EVENT_PARAM = 'invalidEvent';
    
    
    public function configure() {}
    
    
    // @return boolean
    public function filterEvent(MachII_framework_Event $event, MachII_framework_EventContext $eventContext, $paramArgs = array())
    {
        if (!isset($paramArgs[$this->REQUIRED_FIELDS_PARAM])
        || (!isset($paramArgs[$this->INVALID_EVENT_PARAM]))) {
            $error = $this->_throwUsageException();
            return $error;
        }
        
        $requiredFields = explode(',', $paramArgs[$this->REQUIRED_FIELDS_PARAM]);
        $invalidEvent   = $paramArgs[$this->INVALID_EVENT_PARAM];
        
        foreach ($requiredFields as $field) {
            if (!$event->isArgDefined($field)
            || ($event->getArg($field) == false)) {
                $missingFields[] = $field;
            }
        }
        
        if (empty($missingFields)) {
            return true;
        } else {
            $newEventArgs =& $event->getArgs();
            $newEventArgs['message'] = 'Please provide all required fields. Missing fields: '.implode(', ', $missingFields).'.';
            $newEventArgs['missingFields'] = $missingFields;
            $eventContext->announceEvent($invalidEvent, $newEventArgs);
            
            return false;
        }
    }


    // @access private
    private function _throwUsageException()
    {
        $error = MachII::throwError(
            array(
                'className'  => get_class($this),
                'methodName' => $method,
                'message'    => sprintf("RequiredFieldsFilter requires the following usage parameters: %s, %s.", $this->REQUIRED_FIELDS_PARAM, $this->INVALID_EVENT_PARAM)
            ),
            MACHII_ERROR_FILTER
        );
        
        return $error;
    }
    
}

?>