<?php
/**
 * @package MachII
 * @subpackage Commands
 */
/**
 * An EventCommand for processing an EventFilter.
 * @package MachII
 * @subpackage Commands
 */
class MachII_framework_commands_ViewPageCommand extends MachII_framework_EventCommand
{
    /**
     * @access private
     * @var string
     */
    private $_viewName;
    /**
     * @access private
     * @var string a string representation of a variable name
     */
    private $_contentKey;
    
    
    /**
     * @param string
     * @param string a string representation of a variable name
     */
    public function __construct($viewName, $contentKey = null)
    {
        $this->_setViewName($viewName);
        $this->_setContentKey($contentKey);
    }
    
    /**
     * @param MachII_framework_Event or a MachII_framework_Event descendent
     * @param MachII_framework_EventContext
     * @throws MACHII_ERROR
     * @return MACHII_OK
     */
    public function execute(MachII_framework_Event $event, MachII_framework_EventContext $eventContext)
    {
        $error = $eventContext->displayView($event, $this->_getViewName(), $this->_getContentKey());
        if (MachII::isError($error)) {
            return $error;
        }
        
        return MACHII_OK;
    }
    
    /**
     * @access private
     * @param string
     */
    private function _setViewName($viewName)
    {
        $this->_viewName = $viewName;
    }
    
    /**
     * @access private
     * @return string
     */
    private function _getViewName()
    {
        return $this->_viewName;
    }
    
    /**
     * @access private
     * @param string a string representation of a variable name
     */
    private function _setContentKey($contentKey)
    {
        $this->_contentKey = $contentKey;
    }
    
    /**
     * @access private
     * @return string a string representation of a variable name
     */
    private function _getContentKey()
    {
        return $this->_contentKey;
    }
    
    /**
     * @access private
     * @return boolean
     */
    private function _hasContentKey()
    {
        return !is_null($this->_getContentKey);
    }

}

?>