<?php
/**
 * @package MachII
 */
/**
 * Load the Managers
 */
require_once 'MachII/framework/AppManager.php';
require_once 'MachII/framework/PropertyManager.php';
require_once 'MachII/framework/ListenerManager.php';
require_once 'MachII/framework/FilterManager.php';
require_once 'MachII/framework/EventManager.php';
require_once 'MachII/framework/ViewManager.php';
require_once 'MachII/framework/PluginManager.php';

/**
 * Factory class for creating instances of AppManager.
 * @package MachII
 */
class MachII_framework_AppFactory
{
    /**
     * Create an instance of the main application manager
     *
     * Creates an application manager and populates it with the
     * configured component managers.
     * @param string absolute path and filename of the configuration file
     * @return MachII_framework_AppManager
     * @uses MachII_framework_AppManager
     * @uses MachII_framework_PropertyManager
     * @uses MachII_framework_ListenerManager
     * @uses MachII_framework_FilterManager
     * @uses MachII_framework_EventManager
     * @uses MachII_framework_ViewManager
     * @uses MachII_framework_PluginManager
     */
    public function createAppManager($configFile)
    {
        // Configuration
        $config = simplexml_load_file($configFile);
        
        
        //Application Manager
        $appManager = new MachII_framework_AppManager();
        
        // Property Manager
        $appManager->setPropertyManager(
            new MachII_framework_PropertyManager($config, $appManager)
        );
        
        // Listener Manager
        $appManager->setListenerManager(
            new MachII_framework_ListenerManager($config, $appManager)
        );
        
        // Filter Manager
        $appManager->setFilterManager(
            new MachII_framework_FilterManager($config, $appManager)
        );
            
        // Event Manager
        $appManager->setEventManager(
            new MachII_framework_EventManager($config, $appManager)
        );
            
        // View Manager
        $appManager->setViewManager(
            new MachII_framework_ViewManager($config, $appManager)
        );
            
        // Plugin Manager
        $appManager->setPluginManager(
            new MachII_framework_PluginManager($config, $appManager)
        );
        
        $appManager->configure();
        
        return $appManager;
    }

}

?>