<?php
/**
 * @package MachII
 */
/**
 * EventFilter class.
 * @package MachII
 */
abstract class MachII_framework_EventFilter
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    /**
     * @var array associative array of filter parameters
     * @access private
     */
    private $_parameters = array();
    
    
    /**
     * @param MachII_framework_AppManager
     * @param array associative array of event filter parameters
     */
    public function __construct(MachII_framework_AppManager $appManager, $parameters)
    {
        $this->_setAppManager($appManager);
        $this->setParameters($parameters);
    }
    
    /**
     * 
     */
    public function configure() {}
    
    /**
     * @param MachII_framework_Event or descendent
     * @param MachII_framework_EventContext
     * @param array associative array of filter parameters
     * @abstract
     */
    public abstract function filterEvent(MachII_framework_Event $event, MachII_framework_EventContext $eventContext, $paramArgs = array());
    
    /**
     * @access private
     * @param MachII_framework_AppManager
     */
    private function _setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @access protected
     * @return MachII_framework_AppManager
     */
    protected function getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param string
     * @param mixed
     */
    public function setParameter($name, $value)
    {
        $this->_parameters[$name] = $value;
    }
    
    /**
     * @param string
     * @param mixed
     * @return mixed
     */
    public function getParameter($name, $defaultValue = null)
    {
        return (isset($this->_parameters[$name]))?$this->_parameters[$name]:$defaultValue;
    }
    
    /**
     * @param string
     * @return boolean
     */
    public function isParameterDefined($name)
    {
        return isset($this->_parameters[$name]);
    }
    
    /**
     * @param array associative array will be merged with any existing parameters.
     */
    public function setParameters($parameters)
    {
        $this->_parameters = array_merge($this->_parameters, $parameters);
    }
    
    /**
     * @return array associative array
     */
    public function getParameters()
    {
        return $this->_parameters;
    }

}
?>