<?php
/**
 * @package MachII
 */
require_once 'MachII/framework/ListenerInvoker.php';
/**
 * Manages registered listeners.
 * @package MachII
 */
class MachII_framework_ListenerManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of MachII_framework_ListenerManager
     *            descendents.
     * @access private
     */
    private $_listeners = array();
    
    /**
     * @param SimpleXMLElement
     * @param MachII_framework_AppManager
     */
    public function __construct(SimpleXMLElement $configXml, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        
        $listenerParams = array();
        $listeners = $configXml->xpath('//listeners/listener');
        if (empty($listeners)) {
            return;
        }
        foreach ($listeners as $listener) {
            // Set up each listener's parameters
            $parameters = isset($listener->parameters->parameter)?
                $listener->parameters->parameter:
                array();
            $listenerParams = array();
            foreach ($parameters as $parameter) {
                $listenerParams[(string) $parameter['name']] = (string) $parameter['value'];
            }
            
            // Setup the listener's invoker
            $invokerType = MachII_util_ClassLoader::load((string) $listener->invoker['type']);
            $invoker = new $invokerType();
            
            // Listener
            $listenerType = MachII_util_ClassLoader::load((string) $listener['type']);
            
            $listenerObj = new $listenerType($appManager, $listenerParams);
            $listenerObj->setInvoker($invoker);
            
            $this->addListener((string) $listener['name'], $listenerObj);
        }
    }
    
    /**
     * Configure all of the listeners.
     */
    public function configure()
    {
        $keys = array_keys($this->_listeners);
        foreach ($keys as $key) {
            $this->getListener($key)->configure();
        }
    }
    
    /**
     * @param string
     * @param MachII_framework_Listener descendent
     */
    public function addListener($name, MachII_framework_Listener $listener)
    {
        $this->_listeners[$name] = $listener;
    }
    
    /**
     * @param string
     * @return MachII_framework_Listener|false
     */
    public function getListener($listenerName)
    {
        if ($listener = $this->isListenerDefined($listenerName)) {
            return $this->_listeners[$listenerName];
        }
        
        return $listener;
    }
    
    /**
     * @param string
     * @return boolean
     */
    public function isListenerDefined($filterName)
    {
        return isset($this->_listeners[$filterName]);
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }

}

?>