<?php
/**
 * @package MachII
 */
/**
 * Manages registered views.
 * @package MachII
 */
class MachII_framework_ViewManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of filesystem paths
     * @access private
     */
    private $_viewPaths = array();
    
    
    /**
     * @param SimpleXMLElement
     * @param MachII_framework_AppManager
     */
    public function __construct(SimpleXMLElement $configXml, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        
        $views = $configXml->xpath('//page-views/page-view');
        if (empty($views)) {
            return;
        }
        foreach ($views as $view) {
            $this->_viewPaths[(string) $view['name']] = (string) $view['page'];
        }
    }
    
    /**
     * @internal not used at this time 
     */
    public function configure() {}
    
    /**
     * @param string
     * @throws MachII_util_ViewNotDefinedException
     * @return string
     */
    public function getViewPath($viewName)
    {
    //$GLOBALS['vard']->display($this->_viewPaths);
        if ($this->isViewPathDefined($viewName)) {
            //echo 'viewManager->getViewPath: ',$this->_viewPaths[$viewName],'<br />';
            return $this->_viewPaths[$viewName];
        } else {
            throw new MachII_util_ViewNotDefinedException(array('viewName' => $viewName));
        }
    }
    
    /**
     * @param string
     * @return boolean
     */
    public function isViewPathDefined($viewName)
    {
        return isset($this->_viewPaths[$viewName]);
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }

}

?>