<?php
/**
 * @package MachII
 */
/**
 * Plugin.
 * @package MachII
 */
class MachII_framework_Plugin
{
    /**
     * @var MachII_framework_AppManager
     * @access protected
     */
    protected $_appManager;
    
    /**
     * @var array associative array of plugin configuration parameters.
     * @access protected
     */
    protected $_parameters = array();
    
    
    /**
     * @param MachII_framework_AppManager
     * @param array associative array of plugin configuration data.
     */
    public function __construct(MachII_framework_AppManager $appManager, $parameters = array())
    {
        $this->_setAppManager($appManager);
        $this->setParameters($parameters);
    }
    
    /**
     * 
     */
    public function configure() {}
    
    /**
     * Plugin point called when an exception occurs (before exception
     * event is handled).
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     * @param MachII_util_Exception
     */
    public function handleException(MachII_framework_EventContext $eventContext, MachII_util_Exception $exception) {}
    
    /**
     * Aborts event processing
     * 
     * Call this function to abort processing of the current event.
     * When called, a MachII_util AbortEventException exception is
     * thrown, caught, and handled by the framework.
     * @param string 
     * @throws MachII_util_AbortEventException
     */
    public function abortEvent($message = '')
    {
        throw new MachII_util_AbortEventException(array('message' => $message));
    }
    
    /**
     * @param string
     * @param mixed
     */
    public function setParameter($name, $value)
    {
        $this->_parameters[$name] = $value;
    }
    
    /**
     * @param string
     * @param mixed
     * @return mixed
     */
    public function getParameter($name, $defaultValue = null)
    {
        return (isset($this->_parameters[$name]))?
                $this->_parameters[$name]:
                $defaultValue;
    }
    
    /**
     * @param array associative array of plugin parrameters will
     *              be merged with any existing parameters.
     */
    public function setParameters($parameters)
    {
        $this->_parameters = array_merge($this->_parameters, $parameters);
    }
    
    /**
     * @return array associative array of plugin parrameters.
     */
    public function getParameters()
    {
        return $this->_parameters;
    }
    
    /**
     * @access private
     * @param MachII_framework_AppManager
     */
    private function _setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @access protected
     * @return MachII_framework_AppManager
     */
    protected function _getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * Executes once before processing events.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function preProcess(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes before each event.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function preEvent(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes after each event.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function postEvent(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes before each view.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function preView(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes after each view.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function postView(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes once after processing events.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function postProcess(MachII_framework_EventContext $eventContext) {}
    
}

?>