<?php
/**
 * @package Util
 * @subpackage MachII_util_BeanUtil
 */
/**
 * A utility class for working with bean components.
 * 
 * Beans are expected to follow the standard Java bean pattern of having
 * a no argument constuctor and setter functions with name setXXX() (with
 * a single argument named XXX) for field XXX.
 * @package Util
 * @subpackage MachII_util_BeanUtil
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com> PHP Port
 */
class MachII_util_BeanUtil
{
    /**
     * Creates a bean.
     * @param string a fully qualified path to the bean class.
     * @param array an associative array of arguments to pass to the
     *              constructor.
     * @return mixed
     */
    public function createBean($beanType, $beanArgs = array())
    {
        $beanClassName = MachII_util_ClassLoader::load($beanType);
        $bean =& new $beanClassName($beanArgs);
        
        return $bean;
    }
    
    /**
     * Sets the values of fields in a bean.
     * @param mixed
     * @param string a comma delimited list of field names.
     * @param array an associative array of field values.
     * @uses setBeanField()
     */
     public function setBeanFields($bean, $fields, &$fieldCollection)
     {
         $fieldsArray = explode(',', $fields);
         foreach ($fieldsArray as $field) {
             if (isset($fieldCollection[$field])) {
                 $this->setBeanField($bean, $field, $fieldCollection[$field]);
             }
         }
     }
     
     /**
      * Sets the value of a field in a bean.
      * @param mixed
      * @param string
      * @param mixed
      */
     public function setBeanField($bean, $field, &$value)
     {
         $method = 'set'.ucfirst($field);
         $bean->$method($value);
     }

}
?>