<?php
/**
 * @package MachII
 */
/**
 * Mach-II Event
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_Event
{
    /**
     * @var string|void
     * @access private
     */
    private $_name;
    
    /**
     * @var array associative array of event arguments
     * @access private
     */
    private $_args = array();
    
    /**
     * @var string representation of an argument's data type
     * @access private
     */
    private $_argTypes;
        
    /**
     * @param string|void
     * @param array associative array of event arguments
     */
    function __construct($name = null, $args = array())
    {
        $this->_name = $name;
        $this->_args = $args;
    }
    
    /**
     * @param string|void
     */
    public function setName($name)
    {
        $this->_name = $name;
    }
    
    /**
     * @return string|void
     */
    public function getName()
    {
        return $this->_name;
    }
    
    /**
     * @param string
     * @param mixed
     * @param string|void
     */
    public function setArg($name, $value, $type = null)
    {
        $this->_args[$name] = $value;
        
        if (!is_null($type)) {
            $this->_argTypes[$name] = $type;
        }
    }
    
    /**
     * @param string
     * @param mixed
     * @return mixed
     */
    public function getArg($name, $defaultValue = null)
    {
        if (isset($this->_args[$name])) {
            if (is_object($this->_args[$name])) {
                return $this->_args[$name];
            } else {
                $defaultValue = $this->_args[$name];
            }
        }
        
        return $defaultValue;
    }
    
    /**
     * @param string
     * @return boolean
     */
    public function isArgDefined($name)
    {
        return isset($this->_args[$name]);
    }
    
    /**
     * @param string
     */
    public function removeArg($name)
    {
        unset($this->_args[$name]);
    }
    
    /**
     * @param array associative array will be merged with any existing arguments
     */
    public function setArgs($args)
    {
        $keys = array_keys($args);
        foreach ($keys as $name) {
            $this->_args[$name] = $args[$name];
        }
    }
    
    /**
     * @return array associative array
     */
    public function getArgs()
    {
        return $this->_args;
    }
    
    /**
     * @param string
     * @param string|void
     */
    public function setArgType($name, $type)
    {
        $this->_argTypes[$name] = $type;
    }
    
    /**
     * @param string
     * @return string
     */
    public function getArgType($name)
    {
        return isset($this->_argTypes[$name])?$this->_argTypes[$name]:null;
    }
}

?>