<?php
/**
 * @package MachII
 */
/**
 * Handles processing of EventCommands for an Event.
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_EventHandler
{
    /**
     * @var array of MachII_framework_EventCommand descendents
     * @access private
     */
    private $_commands;
    /**
     * Event access.
     * If access is public it can be executed by a user; if private
     * it can only be executed internally.  Defaults to public.
     * @var integer set by constants
     * @see MACHII_EVENT_ACCESS_PUBLIC, MACHII_EVENT_ACCESS_PRIVATE
     */
    private $_access = MACHII_EVENT_ACCESS_PUBLIC;
    
    /**
     * Executes the event's commands.
     * @param MachII_framework_Event or descendent
     * @param MachII_framework_EventContext
     */
    public function handleEvent(MachII_framework_Event $event, MachII_framework_EventContext $eventContext)
    {//echo 'commands: ';
    //echo '<pre>',var_dump($this->_commands),'</pre>';
    //$GLOBALS['vard']->display($this->_commands);
        $eventCnt = count($this->_commands);
        for ($i = 0; $i < $eventCnt; $i++) {
        //echo 'executing command ',$i+1,' of ',count($this->_commands),'<br />';
        //echo 'class: ',get_class($this->_commands[$i]),'<br />';
            $continue = $this->_commands[$i]->execute($event, $eventContext);
            if ($continue == false) {
                break;
            }
        }
    }
    
    /**
     * Adds an event command to the event's command queue.
     * @param MachII_framework_EventCommand or descendent
     */
    public function addCommand(MachII_framework_EventCommand $command)
    {
        $this->_commands[] = $command;
    }
    
    /**
     * @param integer set by a constant
     * @see MACHII_EVENT_ACCESS_PUBLIC, MACHII_EVENT_ACCESS_PRIVATE
     */
    public function setAccess($access)
    {
        $this->_access = $access;
    }
    
    /**
     * @return integer check against constant
     * @see MACHII_EVENT_ACCESS_PUBLIC, MACHII_EVENT_ACCESS_PRIVATE
     */
    public function getAccess()
    {
        return $this->_access;
    }

}
    
?>