<?php
/**
 * @package MachII
 * @subpackage Commands
 */
/**
 * An EventCommand for processing an EventFilter.
 * @package MachII
 * @subpackage Commands
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_commands_NotifyCommand extends MachII_framework_EventCommand
{
    /**
     * @access private
     * @var MachII_framework_Listener
     */
    private $_listener;
    /**
     * @access private
     * @var string
     */
    private $_method;
    /**
     * @access private
     * @var string a string representation of a variable name
     */
    private $_resultKey;
    
    
    /**
     * @param MachII_framework_Listener
     * @param string
     * @param string|void
     */
    public function __construct(MachII_framework_Listener $listener, $method, $resultKey = null)
    {
        $this->_setListener($listener);
        $this->_setMethod($method);
        $this->_setResultKey($resultKey);
    }
    
    /**
     * @param MachII_framework_Event or a MachII_framework_Event descendent
     * @param MachII_framework_EventContext
     * @return true
     */
    public function execute(MachII_framework_Event $event, MachII_framework_EventContext $eventContext)
    {
        $listener = $this->_getListener();
        $invoker  = $listener->getInvoker();
        $invoker->invokeListener($event, $listener, $this->_getMethod(), $this->_getResultKey());
        
        return true;
    }
    
    /**
     * @access private
     * @param MachII_framework_Listener
     */
    private function _setListener(MachII_framework_Listener $listener)
    {
        $this->_listener = $listener;
    }
    
    /**
     * @access private
     * @return MachII_framework_Listener
     */
    private function _getListener()
    {
        return $this->_listener;
    }
    
    /**
     * @access private
     * @param string
     */
    private function _setMethod($method)
    {
        $this->_method = $method;
    }
    
    /**
     * @access private
     * @return string
     */
    private function _getMethod()
    {
        return $this->_method;
    }
    
    /**
     * @access private
     * @param string a string representation of a variable name
     */
    private function _setResultKey($resultKey)
    {
        $this->_resultKey = $resultKey;
    }
    
    /**
     * @access private
     * @return string a string representation of a variable name
     */
    private function _getResultKey()
    {
        return $this->_resultKey;
    }
    
    /**
     * @access private
     * @return boolean
     */
    private function _hasResultKey()
    {
        return !is_null($this->_getResultKey());
    }

}

?>