<?php
/**
 * @package MachII
 */
/**
 * Plugin.
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_Plugin extends MachII_framework_BaseObject
{
    /**
     * @param MachII_framework_AppManager
     * @param array associative array of plugin configuration data.
     */
    public function __construct(MachII_framework_AppManager $appManager, $parameters = array())
    {
        parent::__construct($appManager, $parameters);
    }
    
    /**
     * Override to provide custom configuration logic.
     * 
     * Called after constructor.
     */
    public function configure() {}
    
    /**
     * Plugin point called when an exception occurs (before exception
     * event is handled).
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     * @param MachII_util_Exception
     */
    public function handleException(MachII_framework_EventContext $eventContext, MachII_util_Exception $exception) {}
    
    /**
     * Aborts event processing
     * 
     * Call this function to abort processing of the current event.
     * When called, a MachII_util AbortEventException exception is
     * thrown, caught, and handled by the framework.
     * @param string 
     * @throws MachII_util_AbortEventException
     */
    public function abortEvent($message = '')
    {
        throw new MachII_util_AbortEventException(array('message' => $message));
    }
    
    /**
     * Executes once before processing events.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function preProcess(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes before each event.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function preEvent(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes after each event.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function postEvent(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes before each view.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function preView(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes after each view.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function postView(MachII_framework_EventContext $eventContext) {}
    
    /**
     * Executes once after processing events.
     * 
     * Override to provide custom functionality.
     * @param MachII_framework_EventContext
     */
    public function postProcess(MachII_framework_EventContext $eventContext) {}
    
}

?>