<?php
/**
 * @package MachII
 */
/**
 * Handles view display for a  {@link MachII_framework_EventContext}.
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_ViewContext
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    
    /**
     * @param MachII_framework_AppManager
     */
    public function __construct(MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
    }
    
    /**
     * @param MachII_framework_Event
     * @param string
     * @param string|void string representation of a variable name.
     * @param boolean whether or not to append output to the content key.
     */
    public function displayView(MachII_framework_Event $event, $viewName, $contentKey = null, $append = false)
    {//echo 'ViewContext->displayView - $viewName: ',$viewName,'<br />';
        $GLOBALS['event'] = $event;
        $viewPath = $this->_getFullPath($viewName);
        //echo $viewPath,'<br />';
        if (!empty($contentKey)) {
            ob_start();
            include $viewPath;
            $content = ob_get_clean();            
            
            if ($content === false) {
                $content = '';
            }
            
            if (strstr($contentKey, '[')) {
                // Create content container
                eval("return \$$contentKey = ((\$append) && (isset(\$$contentKey)))?
                    \$$contentKey.\$content:
                    \$content;");
            } else {
                if (($append)
                && ($event->isArgDefined($contentKey))) {
                    $content = $event->getArg($contentKey).$content;
                }
                $event->setArg($contentKey, $content);
            }
        } else {
            include $viewPath;
        }
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param MachII_framework_AppManager
     * @access private
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * Gets the specified property.
     * 
     * This is just a shortcut for getAppManager()->getPropertyManager()->getProperty().
     * @param string The name of the property to return.
     * @return mixed The value of a property.
     */
    public function getProperty($propertyName)
    {
        if (!isset($this->_propertyManager)) {
            $this->_propertyManager = $this->getAppManager()->getPropertyManager();
        }
        
        return $this->_propertyManager->getProperty($propertyName);
    }
    
    /**
     * Sets the specified property.
     * 
     * This is just a shortcut for getAppManager()->getPropertyManager()->setProperty().
     * @param string The name of the property to set.
     * @param mixed The value to store in the property.
     * @return mixed
     */
    public function setProperty($propertyName, $propertyValue)
    {
        if (!isset($this->_propertyManager)) {
            $this->_propertyManager = $this->getAppManager()->getPropertyManager();
        }
        
        return $this->_propertyManager->setProperty($propertyName, $propertyValue);
    }
    
    /**
     * @param string
     * @return string
     * @access private
     */
    private function _getFullPath($viewName)
    {
        $viewPath = $this->getAppManager()->getViewManager()->getViewPath($viewName);
        
        $appRoot = $this->_getAppRoot();
        
        if ($appRoot{0} == '/') {
            $webRoot = MachII_util_ClassLoader::getWebRoot();
            if (file_exists($webRoot.$appRoot.$viewPath)) {
                return $webRoot.$appRoot.$viewPath;
            }
        }
        
        return $appRoot.$viewPath;
    }
    
    /**
     * @return string the defined application root
     * @access private
     */
    private function _getAppRoot()
    {
        return $this->getProperty('applicationRoot');
    }

}

?>