<?php
/**
 * @package MachII
 * @subpackage Commands
 */
/**
 * An EventCommand for creating and populating a bean in the current event.
 * @package MachII
 * @subpackage Commands
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_commands_EventBeanCommand extends MachII_framework_EventCommand
{
    /**
     * @access private
     * @var string
     */
    private $_beanName;
    /**
     * @access private
     * @var string
     */
    private $_beanType;
    /**
     * @access private
     * @var string
     */
    private $_beanFields;
    /**
     * @access private
     * @var MachII_util_BeanUtil
     */
    private $_beanUtil;
    
    /**
     * @param string
     * @param string
     * @param string a comma delimited list
     */
    public function __construct($beanName, $beanType, $beanFields)
    {
        $this->_setBeanName($beanName);
        $this->_setBeanType($beanType);
        $this->_setBeanFields($beanFields);
        
        $className = MachII_util_ClassLoader::load('MachII.util.BeanUtil');
        $this->_setBeanUtil(new $className());
    }
    
    /**
     * @param MachII_framework_Event MachII_framework_Event descendent
     * @param MachII_framework_EventContext
     * @return true
     */
    public function execute(MachII_framework_Event $event, MachII_framework_EventContext $eventContext)
    {
        $beanType = $this->_getBeanType();
        
        if ($this->isBeanFieldsDefined()) {
            $bean = $this->_getBeanUtil()->createBean($beanType);
            $this->_getBeanUtil()->setBeanFields($bean, $this->getBeanFields(), $event->getArgs());
        } else {
            $bean = $this->_getBeanUtil()->createBean($beanType, $event->getArgs());
        }
        
        $event->setArg($this->_getBeanName(), $bean, $beanType);
        
        return true;
    }
    
    /**
     * @return boolean
     */
    public function isBeanFieldsDefined()
    {
        return isset($this->_beanFields);
    }
    
    /**
     * @param string
     * @access private
     */
    private function _setBeanName($beanName)
    {
        $this->_beanName = $beanName;
    }
    
    /**
     * @return string
     * @access private
     */
    private function _getBeanName()
    {
        return $this->_beanName;
    }
    
    /**
     * @param string
     * @access private
     */
    private function _setBeanType($beanType)
    {
        $this->_beanType = $beanType;
    }
    
    /**
     * @return string
     * @access private
     */
    private function _getBeanType()
    {
        return $this->_beanType;
    }
    
    /**
     * @param string a comma delimited list of field names
     * @access private
     */
    private function _setBeanFields($beanFields)
    {
        $this->_beanFields = $beanFields;
    }
    
    /**
     * @return string a comma delimited list of field names
     * @access private
     */
    private function _getBeanFields()
    {
        return $this->_beanFields;
    }
    
    /**
     * @param MachII_util_BeanUtil
     * @access private
     */
    private function _setBeanUtil($beanUtil)
    {
        $this->_beanUtil = $beanUtil;
    }
    
    
    /**
     * @return MachII_util_BeanUtil
     * @access private
     */
    private function _getBeanUtil()
    {
        return $this->_beanUtil;
    }

}

?>