<?php
/**
 * @package MachII
 */
/**
 * Load base event filter class
 */
require_once 'MachII/framework/EventFilter.php';
/**
 * Manages registered {@link MachII_framework_EventFilter}s.
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_FilterManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of MachII_framework_EventFilter descendents
     * @access private
     */
    private $_filters = array();
    
    
    /**
     * @param SimpleXMLElement
     * @param MachII_framework_AppManager
     */
    public function __construct(SimpleXMLElement $configXml, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        
        $filters = $configXml->xpath('//event-filters/event-filter');
        if (empty($filters)) {
            return;
        }
        foreach ($filters as $filter) {
            $parameters = isset($filter->parameter)?
                $filter->parameter:
                array();
            $filterParams = array();
            
            foreach ($parameters as $parameter) {
                $filterParams[(string) $parameter['name']] = (string) $parameter['value'];
            }
            
            $filterType = MachII_util_ClassLoader::load((string) $filter['type']);
            
            $this->addFilter((string) $filter['name'], new $filterType($appManager, $filterParams));
        }
    }
    
    /**
     * Configure all of the filters
     */
    public function configure()
    {
        $keys = array_keys($this->_filters);
        foreach ($keys as $key) {
            $filter = $this->getFilter($key)->configure();
        }
    }
    
    /**
     * @param string
     * @param MachII_framework_EventFilter or descendent
     */
    public function addFilter($name, MachII_framework_EventFilter $filter)
    {
        $this->_filters[$name] = $filter;
    }
    
    /**
     * @param string
     * @throws MachII_util_EventFilterNotDefined
     * @return MachII_framework_EventFilter or descendent
     */
    public function getFilter($filterName)
    {
        if ($this->isFilterDefined($filterName)) {
            return $this->_filters[$filterName];
        } else {
            throw new MachII_util_EventFilterNotDefinedException(array('filterName' => $filterName));
        }
    }
    
    /**
     * @param string
     * @return MachII_framework_EventFilter or descendent
     */
    public function isFilterDefined($filterName)
    {
        return isset($this->_filters[$filterName]);
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }

}

?>