<?php
/**
 * @package MachII
 * @subpackage Invokers
 */
/**
 * A MachII_framework_Invoker for calling a MachII_framework_Listener.
 * 
 * Event arguments will be mapped to MachII_framework_Listener method
 * arguments via a manually created argument map.
 * @see MachII_framework_Listener::__dispatch_map
 * @package MachII
 * @subpackage Invokers
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_invokers_ObjectEventArgs extends MachII_framework_ListenerInvoker
{
    /**
     * {@inheritdoc}
     * @uses MachII_framework_Listener::__dispatch()
     */
    public function invokeListener(MachII_framework_Event $event, MachII_framework_Listener $target, $method, $resultKey = null)
    {
        try {
            //$resultKeyRef = (isset($resultKey))?$resultKey:'resultKeyRef';
            
            // Get the map of inputs for the method being called
            $dispatchMap = $target->__dispatch($method);
            if (!$dispatchMap) {
                throw new MachII_util_ListenerInvokerDispatchMapNotDefinedException(
                    array(
                        'className'  => get_class($target),
                        'methodName' => $method
                    )
                );
            } // TODO: maybe use a tokenizer to build map at parse time?
            
            // Build the arg list
            $args = array();
            $eventArgs = $event->getArgs();
            
            $dispatchMapArgs = array_keys($dispatchMap['in']);
            
            foreach ($dispatchMapArgs as $key=>$arg) {
                if (isset($eventArgs[$arg])) {
                    $args[] = '$eventArgs[$dispatchMapArgs['.$key.']]';
                }
            }
            if (count($dispatchMap['in']) != count($args)) {
                throw new MachII_util_ListenerInvokerArgCountMismatchException(
                    array(
                        'className'       => get_class($target),
                        'methodName'      => $method,
                        'argsFoundNum'    => count($args),
                        'argsExpectedNum' => count($dispatchMap['in'])
                    )
                );
            }
            
            // Invoke the listener
            $result = call_user_func_array(array($target, $method), $args);
            
            if ((isset($resultKey)) {
                if (strstr($resultKey, '['))) {
                    eval("return \$$resultKeyRef = $result;")
                } else {
                    $event->setArg($resultKey, $result);
                }
            }
        
        } catch (Exception $e) {
            throw $e;
        }
    }

}

?>