<?php
/**
 * @package Cache
 * @subpackage MachII_cache_DangaMemCachedClient
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
 
// +----------------------------------------------------------------------+
// | Mach-II :: MachII_cache_DangaMemCachedClient                         |
// +----------------------------------------------------------------------+
// | Copyright (c) 2003-2004 Alan Richmond                                |
// | All rights reserved.                                                 |
// +----------------------------------------------------------------------+
// | Redistribution and use in source and binary forms, with or without   |
// | modification, are permitted provided that the following conditions   |
// | are met:                                                             |
// |                                                                      |
// | Redistributions of source code must retain the above copyright       |
// | notice, this list of conditions and the following disclaimer.        |
// |                                                                      |
// | Redistributions in binary form must reproduce the above copyright    |
// | notice, this list of conditions and the following disclaimer in the  |
// | documentation and/or other materials provided with the distribution. |
// |                                                                      |
// | Neither the name of Alan Richmond nor the names of his contributors  |
// | may be used to endorse or promote products derived from this         |
// | software without specific prior written permission.                  |
// |                                                                      |
// | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  |
// | "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    |
// | LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS    |
// | FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE      |
// | REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,          |
// | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, |
// | BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS|
// | OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED   |
// | AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT          |
// | LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY|
// | WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          |
// | POSSIBILITY OF SUCH DAMAGE.                                          |
// +----------------------------------------------------------------------+

/**
 * Mach-II cache adapter for Danga MemCachedClient.
 * 
 * Mach-II cache adapter for {@link http://www.danga.com/memcached/dist/php-memcached-1.0.10.tar.gz MemCachedClient}
 * a PHP API to Danga Interactive's {@link http://www.danga.com/memcached/ memcached},
 * a high-performance, distributed memory object caching system.
 * The file <code>MemCachedClient.inc.php</code> must be installed and
 * memcached daemon must be running before this adapter may be used.
 * @package Cache
 * @subpackage MachII_cache_DangaMemCachedClient
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_cache_DangaMemCachedClient extends MachII_framework_Cache
{
    /**
     * Constructor
     * 
     * Set some defaults and get the cache ready for use.
     * 
     * Any valid MemCachedClient option may be passed in through the
     * 'options' array.  See the memcached-client source for more details.
     * The following defaults are set:
     * <ul>
     *    <li>servers = 127.0.0.1:11211</li>
     *    <li>debug = false</li>
     *    <li>compress = true</li>
     * </ul>
     * @uses MachII_util_Cache::__construct()
     * @param string a name unique to this application
     * @param array an associative array of memcached-client options
     */
    function MachII_cache_DangaMemCachedClient($appName, $options = array())
    {
        if (!isset($options['servers'])) {
            $options['servers'] = array('127.0.0.1:11211');
        }
        if (!isset($options['debug'])) {
            $options['debug'] = false;
        }
        if (!isset($options['compress'])) {
            $options['compress'] = true;
        }
        
        parent::__construct($appName, $options);
    }
    
    /**
     * @param string
     * @param mixed
     * @uses MemCachedClient::set()
     * @return boolean
     */
    function save($name, $value)
    {
        $result = $this->cache->set($this->_makeName($name), $value);
        
        return $result;
    }
    
    /**
     * @param string
     * @uses MemCachedClient::get()
     * @return boolean
     */
    function &get($name)
    {
        $value = $this->cache->get($this->_makeName($name));
        
        return $value;
    }
    
    /**
     * Clear application variables.
     * 
     * MemCachedClient::set() always replaces by default so vars don't
     * need to be cleared befor over-writing.
     */
    function clear() {}
    
    /**
     * Instantiate a new MemCachedClient object.
     * @uses MemCachedClient
     * @return MemCachedClient
     */
    function &createCache()
    {
        require_once 'MemCachedClient.inc.php';
    
        $cache =& new MemCachedClient($this->options);
        
        return $cache;
    }
    
    /**
     * Creates a variable name in the format of appName.varName so
     * multiple apps can run on memcached.
     * @param string
     * @return string
     */
    function _makeName($name)
    {
        return $this->appName.'.'.$name;
    }
}

?>
