<?php
/**
 * @package Cache
 * @subpackage MachII_cache_TurckMmcache
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
 
// +----------------------------------------------------------------------+
// | Mach-II :: MachII_cache_TurckMmcache                                 |
// +----------------------------------------------------------------------+
// | Copyright (c) 2004,2005 Alan Richmond                                |
// | All rights reserved.                                                 |
// +----------------------------------------------------------------------+
// | Redistribution and use in source and binary forms, with or without   |
// | modification, are permitted provided that the following conditions   |
// | are met:                                                             |
// |                                                                      |
// | Redistributions of source code must retain the above copyright       |
// | notice, this list of conditions and the following disclaimer.        |
// |                                                                      |
// | Redistributions in binary form must reproduce the above copyright    |
// | notice, this list of conditions and the following disclaimer in the  |
// | documentation and/or other materials provided with the distribution. |
// |                                                                      |
// | Neither the name of Alan Richmond nor the names of his contributors  |
// | may be used to endorse or promote products derived from this         |
// | software without specific prior written permission.                  |
// |                                                                      |
// | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  |
// | "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    |
// | LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS    |
// | FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE      |
// | REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,          |
// | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, |
// | BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS|
// | OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED   |
// | AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT          |
// | LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY|
// | WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          |
// | POSSIBILITY OF SUCH DAMAGE.                                          |
// +----------------------------------------------------------------------+

/**
 * Mach-II cache facade for Turck MMCache.
 * 
 * Mach-II cache facade for {@link http://turck-mmcache.sourceforge.net/ Turck MMCache},
 * a free open source PHP accelerator, optimizer, encoder and dynamic
 * content cache.for Unix and Windows.  The MMCache module (.so or .dll)
 * must be installed and configured before this facade may be used.
 * 
 * <caution>Note:</caution> Turck MMCache development has been continued as
 * {@link http://eaccelerator.sourceforge.net/ eAccelerator}.  You should
 * probably switch, as eAccelerator is being activly maintained.
 * @package Cache
 * @subpackage MachII_cache_TurckMmcache
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_cache_TurckMmcache extends MachII_framework_Cache
{
    /**
     * Constructor
     * 
     * Set some defaults and get the cache ready for use.
     * 
     * @uses MachII_util_Cache::__construct
     * @param string a name unique to this application
     * @param array an associative array of options
     */
    function MachII_cache_TurckMmcache($appName, $options = array())
    {
        parent::__construct($appName, $options);
    }
    
    /**
     * @param string
     * @param mixed
     * @uses MemCachedClient::set()
     * @return boolean
     */
    function save($name, $value)
    {
        $name = $this->_makeName($name);
        
        mmcache_lock($name);
        $result = mmcache_put($name, $value);
        mmcache_unlock($name);
        
        return $result;
    }
    
    /**
     * @param string
     * @uses MemCachedClient::get()
     * @return boolean
     */
    function &get($name)
    {
        $name = $this->_makeName($name);
        
        mmcache_lock($name);
        $value = mmcache_get($name);
        mmcache_unlock($name);
        
        return $value;
    }
    
    /**
     * Clear application variables.
     */
    function clear() {}
    
    /**
     * Instantiate a new MemCachedClient object.
     * @uses MemCachedClient
     * @return MemCachedClient
     */
    function &createCache()
    {
        return true;
    }
    
    /**
     * Creates a variable name in the format of appName.varName so
     * multiple apps can run on MMCache.
     * @param string
     * @return string
     * @access private
     */
    function _makeName($name)
    {
        return $this->appName.'.'.$name;
    }
}

?>
