<?php

class Roulette_model_RouletteTable extends MachII_framework_Listener
{
    private $_player;
    private $_bet;
    private $_history;
    
    public function configure()
    {
        MachII_util_ClassLoader::load('Roulette.model.Bet');
        MachII_util_ClassLoader::load('Roulette.model.History');
        
        $this->_bet     = new Roulette_model_Bet();
        $this->_history = new Roulette_model_History();
    }
    
    public function newPlayer($event)
    {
        $playerClass = MachII_util_ClassLoader::load('Roulette.model.Person');
        
        $this->_player = new $playerClass($event->getArg('playerName'), $event->getArg('stake'));
        $this->_bet->clearBet();
    }
    
    public function getPlayer()
    {
        return $this->_player;
    }
    
    public function getBet()
    {
        return $this->_bet;
    }
    
    public function getHistory()
    {
        return $this->_history;
    }
    
    public function getGameState()
    {
        $state = array(
            'betAmount'  => $this->getBet()->getAmountBet(),
            'history'    => $this->getHistory()->getWinners(),
            'stake'      => $this->getPlayer()->getBankAccount()->getBankAmount(),
            'playerName' => $this->getPlayer()->getName()
        );
        
        return $state;
    }
    
    public function placeBet($event)
    {
        $amountBet  = $event->getArg('amountBet');
        $numbersBet = $event->getArg('numbersBet');
        
        $account = $this->getPlayer()->getBankAccount();
        
        if ((count($numbersBet) * $amountBet) <= $account) {
            $this->getBet()->setAmountBet($amountBet);
            $this->getBet()->setNumbersBet($numbersBet);
            $this->announceEvent('acceptedBet');
        } else {
            $this->rejectBet();
        }
    }
    
    public function settleGame()
    {
        $winner = $this->generateWinningNumber();
        $numbersBet = $this->getBet()->getNumbersBet();
        
        $this->getHistory()->addToHistory($winner);
        
        if (in_array($winner, $numbersBet)) {
            $this->playerWins();
        } else {
            $this->playerLoses();
        }
    }
    
    public function playerWins()
    {
        $this->getPlayer()->getBankAccount()->creditAccount($this->getBet()->getAmountBet() * 37);
    }
    
    public function playerLoses()
    {
        $numberOfBets = count($this->getBet()->getNumbersBet());
        
        $this->getPlayer()->getBankAccount()->debitAccount($this->getBet()->getAmountBet() * $numberOfBets);
    }
    
    public function rejectBet()
    {
        $this->announceEvent('rejectedBet', array('message' => 'You don\'t have enough to place that bet.'));
    }
    
    public function generateWinningNumber()
    {
        $winner = mt_rand(1, 38);
        
        if ($winner == 37) {
            return '0';
        } elseif ($winner == 38) {
            return '00';
        }
        
        return $winner;
    }
}

?>